@@ -110,6 +110,9 @@ typedef uint8_t pmix_cmd_t;
 /* provide a "pretty-print" function for cmds */
 const char* pmix_command_string(pmix_cmd_t cmd);
 
+/* provide a hook to init tool data */
+PMIX_EXPORT extern pmix_status_t pmix_tool_init_info(void);
+
 /* define a set of flags to direct collection
  * of data during operations */
 typedef enum {
@@ -119,6 +122,16 @@ typedef enum {
     PMIX_COLLECT_MAX
 } pmix_collect_t;
 
+/* define a set of flags indicating the level
+ * of information being stored/requested */
+typedef enum {
+    PMIX_LEVEL_UNDEF,
+    PMIX_LEVEL_SESSION,
+    PMIX_LEVEL_JOB,
+    PMIX_LEVEL_APP,
+    PMIX_LEVEL_NODE
+} pmix_level_t;
+
 /****    PEER STRUCTURES    ****/
 
 /* clients can only talk to their server, and servers are
@@ -265,6 +278,7 @@ typedef struct {
     pmix_info_t *info;
     size_t ninfo;
     pmix_byte_object_t bo;
+    pmix_list_t results;
     pmix_info_cbfunc_t cbfunc;
     pmix_value_cbfunc_t valcbfunc;
     pmix_release_cbfunc_t relcbfunc;
@@ -280,7 +294,7 @@ typedef struct {
     pmix_list_item_t super;
     pmix_event_t ev;
     bool event_active;
-    bool lost_connection;           // tracker went thru lost connection procedure
+    bool host_called;               // tracker has been passed up to host
     bool local;                     // operation is strictly local
     char *id;                       // string identifier for the collective
     pmix_cmd_t type;
@@ -385,6 +399,7 @@ typedef struct {
     pmix_list_t kvs;
     bool copy;
     bool timer_running;
+    pmix_level_t level;
 } pmix_cb_t;
 PMIX_CLASS_DECLARATION(pmix_cb_t);
 
@@ -478,6 +493,7 @@ typedef struct {
     bool xml_output;
     bool timestamp_output;
     size_t output_limit;
+    pmix_list_t nspaces;
 } pmix_globals_t;
 
 /* provide access to a function to cleanup epilogs */
