@@ -1,6 +1,6 @@
 /* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
- * Copyright (c) 2015-2018 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2015-2019 Intel, Inc.  All rights reserved.
  * Copyright (c) 2016      Mellanox Technologies, Inc.
  *                         All rights reserved.
  * Copyright (c) 2018      Research Organization for Information Science
@@ -39,10 +39,11 @@ pmix_status_t pmix_pnet_base_allocate(char *nspace,
                                       pmix_list_t *ilist)
 {
     pmix_pnet_base_active_module_t *active;
-    pmix_status_t rc;
-    pmix_nspace_t *nptr, *ns;
+    pmix_status_t rc = PMIX_SUCCESS;
+    pmix_namespace_t *nptr, *ns;
     size_t n;
     char *nregex, *pregex;
+    char *params[2] = {"PMIX_MCA_", NULL};
 
     if (!pmix_pnet_globals.initialized) {
         return PMIX_ERR_INIT;
@@ -59,7 +60,7 @@ pmix_status_t pmix_pnet_base_allocate(char *nspace,
         nptr = NULL;
         /* find this nspace - note that it may not have
          * been registered yet */
-        PMIX_LIST_FOREACH(ns, &pmix_server_globals.nspaces, pmix_nspace_t) {
+        PMIX_LIST_FOREACH(ns, &pmix_globals.nspaces, pmix_namespace_t) {
             if (0 == strcmp(ns->nspace, nspace)) {
                 nptr = ns;
                 break;
@@ -67,30 +68,15 @@ pmix_status_t pmix_pnet_base_allocate(char *nspace,
         }
         if (NULL == nptr) {
             /* add it */
-            nptr = PMIX_NEW(pmix_nspace_t);
+            nptr = PMIX_NEW(pmix_namespace_t);
             if (NULL == nptr) {
                 return PMIX_ERR_NOMEM;
             }
             nptr->nspace = strdup(nspace);
-            pmix_list_append(&pmix_server_globals.nspaces, &nptr->super);
+            pmix_list_append(&pmix_globals.nspaces, &nptr->super);
         }
 
-        /* if the info param is NULL, then we make one pass thru the actives
-         * in case someone specified an allocation or collection of envars
-         * via MCA param */
-        if (NULL == info) {
-            PMIX_LIST_FOREACH(active, &pmix_pnet_globals.actives, pmix_pnet_base_active_module_t) {
-                if (NULL != active->module->allocate) {
-                    if (PMIX_SUCCESS == (rc = active->module->allocate(nptr, NULL, ilist))) {
-                        break;
-                    }
-                    if (PMIX_ERR_TAKE_NEXT_OPTION != rc) {
-                        /* true error */
-                        return rc;
-                    }
-                }
-            }
-        } else {
+        if (NULL != info) {
             /* check for description of the node and proc maps */
             nregex = NULL;
             pregex = NULL;
@@ -131,7 +117,10 @@ pmix_status_t pmix_pnet_base_allocate(char *nspace,
         }
     }
 
-    return PMIX_SUCCESS;
+    /* add any local PMIx MCA params */
+    rc = pmix_pnet_base_harvest_envars(params, NULL, ilist);
+
+    return rc;
 }
 
 /* can only be called by a server */
@@ -141,7 +130,7 @@ pmix_status_t pmix_pnet_base_setup_local_network(char *nspace,
 {
     pmix_pnet_base_active_module_t *active;
     pmix_status_t rc;
-    pmix_nspace_t *nptr, *ns;
+    pmix_namespace_t *nptr, *ns;
 
     if (!pmix_pnet_globals.initialized) {
         return PMIX_ERR_INIT;
@@ -157,7 +146,7 @@ pmix_status_t pmix_pnet_base_setup_local_network(char *nspace,
 
     /* find this proc's nspace object */
     nptr = NULL;
-    PMIX_LIST_FOREACH(ns, &pmix_server_globals.nspaces, pmix_nspace_t) {
+    PMIX_LIST_FOREACH(ns, &pmix_globals.nspaces, pmix_namespace_t) {
         if (0 == strcmp(ns->nspace, nspace)) {
             nptr = ns;
             break;
@@ -165,12 +154,12 @@ pmix_status_t pmix_pnet_base_setup_local_network(char *nspace,
     }
     if (NULL == nptr) {
         /* add it */
-        nptr = PMIX_NEW(pmix_nspace_t);
+        nptr = PMIX_NEW(pmix_namespace_t);
         if (NULL == nptr) {
             return PMIX_ERR_NOMEM;
         }
         nptr->nspace = strdup(nspace);
-        pmix_list_append(&pmix_server_globals.nspaces, &nptr->super);
+        pmix_list_append(&pmix_globals.nspaces, &nptr->super);
     }
 
     PMIX_LIST_FOREACH(active, &pmix_pnet_globals.actives, pmix_pnet_base_active_module_t) {
@@ -189,7 +178,7 @@ pmix_status_t pmix_pnet_base_setup_fork(const pmix_proc_t *proc, char ***env)
 {
     pmix_pnet_base_active_module_t *active;
     pmix_status_t rc;
-    pmix_nspace_t *nptr, *ns;
+    pmix_namespace_t *nptr, *ns;
 
     if (!pmix_pnet_globals.initialized) {
         return PMIX_ERR_INIT;
@@ -202,7 +191,7 @@ pmix_status_t pmix_pnet_base_setup_fork(const pmix_proc_t *proc, char ***env)
 
     /* find this proc's nspace object */
     nptr = NULL;
-    PMIX_LIST_FOREACH(ns, &pmix_server_globals.nspaces, pmix_nspace_t) {
+    PMIX_LIST_FOREACH(ns, &pmix_globals.nspaces, pmix_namespace_t) {
         if (0 == strcmp(ns->nspace, proc->nspace)) {
             nptr = ns;
             break;
@@ -210,17 +199,18 @@ pmix_status_t pmix_pnet_base_setup_fork(const pmix_proc_t *proc, char ***env)
     }
     if (NULL == nptr) {
         /* add it */
-        nptr = PMIX_NEW(pmix_nspace_t);
+        nptr = PMIX_NEW(pmix_namespace_t);
         if (NULL == nptr) {
             return PMIX_ERR_NOMEM;
         }
         nptr->nspace = strdup(proc->nspace);
-        pmix_list_append(&pmix_server_globals.nspaces, &nptr->super);
+        pmix_list_append(&pmix_globals.nspaces, &nptr->super);
     }
 
     PMIX_LIST_FOREACH(active, &pmix_pnet_globals.actives, pmix_pnet_base_active_module_t) {
         if (NULL != active->module->setup_fork) {
-            if (PMIX_SUCCESS != (rc = active->module->setup_fork(nptr, proc, env))) {
+            rc = active->module->setup_fork(nptr, proc, env);
+            if (PMIX_SUCCESS != rc && PMIX_ERR_NOT_AVAILABLE != rc) {
                 return rc;
             }
         }
@@ -252,7 +242,7 @@ void pmix_pnet_base_child_finalized(pmix_proc_t *peer)
     return;
 }
 
-void pmix_pnet_base_local_app_finalized(pmix_nspace_t *nptr)
+void pmix_pnet_base_local_app_finalized(pmix_namespace_t *nptr)
 {
     pmix_pnet_base_active_module_t *active;
 
@@ -277,7 +267,9 @@ void pmix_pnet_base_local_app_finalized(pmix_nspace_t *nptr)
 void pmix_pnet_base_deregister_nspace(char *nspace)
 {
     pmix_pnet_base_active_module_t *active;
-    pmix_nspace_t *nptr, *ns;
+    pmix_namespace_t *nptr, *ns;
+    pmix_pnet_job_t *job;
+    pmix_pnet_node_t *node;
 
     if (!pmix_pnet_globals.initialized) {
         return;
@@ -290,7 +282,7 @@ void pmix_pnet_base_deregister_nspace(char *nspace)
 
     /* find this nspace object */
     nptr = NULL;
-    PMIX_LIST_FOREACH(ns, &pmix_server_globals.nspaces, pmix_nspace_t) {
+    PMIX_LIST_FOREACH(ns, &pmix_globals.nspaces, pmix_namespace_t) {
         if (0 == strcmp(ns->nspace, nspace)) {
             nptr = ns;
             break;
@@ -307,7 +299,24 @@ void pmix_pnet_base_deregister_nspace(char *nspace)
         }
     }
 
-    return;
+    PMIX_LIST_FOREACH(job, &pmix_pnet_globals.jobs, pmix_pnet_job_t) {
+        if (0 == strcmp(nspace, job->nspace)) {
+            pmix_list_remove_item(&pmix_pnet_globals.jobs, &job->super);
+            PMIX_RELEASE(job);
+            break;
+        }
+    }
+
+    PMIX_LIST_FOREACH(node, &pmix_pnet_globals.nodes, pmix_pnet_node_t) {
+        pmix_pnet_local_procs_t *lp;
+        PMIX_LIST_FOREACH(lp, &node->local_jobs, pmix_pnet_local_procs_t) {
+            if (0 == strcmp(nspace, lp->nspace)) {
+                pmix_list_remove_item(&node->local_jobs, &lp->super);
+                PMIX_RELEASE(lp);
+                break;
+            }
+        }
+    }
 }
 
 static void cicbfunc(pmix_status_t status,
@@ -560,6 +569,8 @@ pmix_status_t pmix_pnet_base_harvest_envars(char **incvars, char **excvars,
                 }
                 *string_key = '\0';
                 ++string_key;
+                pmix_output_verbose(5, pmix_pnet_base_framework.framework_output,
+                                    "pnet: adding envar %s", cs_env);
                 PMIX_ENVAR_LOAD(&kv->value->data.envar, cs_env, string_key, ':');
                 pmix_list_append(ilist, &kv->super);
                 free(cs_env);
@@ -576,6 +587,8 @@ pmix_status_t pmix_pnet_base_harvest_envars(char **incvars, char **excvars,
             }
             PMIX_LIST_FOREACH_SAFE(kv, next, ilist, pmix_kval_t) {
                 if (0 == strncmp(kv->value->data.envar.envar, excvars[j], len)) {
+                pmix_output_verbose(5, pmix_pnet_base_framework.framework_output,
+                                    "pnet: excluding envar %s", kv->value->data.envar.envar);
                     pmix_list_remove_item(ilist, &kv->super);
                     PMIX_RELEASE(kv);
                 }
