@@ -24,6 +24,7 @@
  */
 
 #include <src/include/pmix_config.h>
+#include "src/include/pmix_globals.h"
 
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
@@ -50,7 +51,6 @@
 #include <sys/sysctl.h>
 #endif
 
-#include "src/include/pmix_globals.h"
 #include "src/include/pmix_socket_errno.h"
 #include "src/client/pmix_client_ops.h"
 #include "src/server/pmix_server_ops.h"
@@ -131,7 +131,7 @@ static pmix_status_t connect_to_peer(struct pmix_peer_t *peer,
     char *p, *p2, *server_nspace = NULL, *rendfile = NULL;
     int sd, rc;
     size_t n;
-    char myhost[PMIX_MAXHOSTNAMELEN];
+    char myhost[PMIX_MAXHOSTNAMELEN] = {0};
     bool system_level = false;
     bool system_level_only = false;
     bool reconnect = false;
@@ -414,7 +414,7 @@ static pmix_status_t connect_to_peer(struct pmix_peer_t *peer,
 
     /* mark that we are using the V2 protocol */
     pmix_globals.mypeer->protocol = PMIX_PROTOCOL_V2;
-    gethostname(myhost, sizeof(myhost));
+    gethostname(myhost, sizeof(myhost)-1);
     /* if we were given a URI via MCA param, then look no further */
     if (NULL != suri) {
         if (NULL != server_nspace) {
@@ -1411,7 +1411,18 @@ static pmix_status_t recv_connect_ack(int sd, uint8_t myflag)
                             pmix_client_globals.myserver->info->pname.rank);
 
         /* get the returned status from the security handshake */
-        pmix_ptl_base_recv_blocking(sd, (char*)&reply, sizeof(pmix_status_t));
+        rc = pmix_ptl_base_recv_blocking(sd, (char*)&u32, sizeof(pmix_status_t));
+        if (PMIX_SUCCESS != rc) {
+            if (sockopt) {
+                /* return the socket to normal */
+                if (0 != setsockopt(sd, SOL_SOCKET, SO_RCVTIMEO, &save, sz)) {
+                    return PMIX_ERR_UNREACH;
+                }
+            }
+            return rc;
+        }
+
+        reply = ntohl(u32);
         if (PMIX_SUCCESS != reply) {
             /* see if they want us to do the handshake */
             if (PMIX_ERR_READY_FOR_HANDSHAKE == reply) {
