@@ -15,7 +15,8 @@
  * Copyright (c) 2016-2019 Intel, Inc.  All rights reserved.
  * Copyright (c) 2017-2018 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
- * Copyright (c) 2018      IBM Corporation.  All rights reserved.
+ * Copyright (c) 2018-2019 IBM Corporation.  All rights reserved.
+ * Copyright (c) 2019      Mellanox Technologies, Inc. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -117,6 +118,7 @@ static pmix_status_t setup_fork(const pmix_proc_t *proc, char ***env);
     .disable_ipv6_family = true,
     .session_filename = NULL,
     .nspace_filename = NULL,
+    .pid_filename = NULL,
     .system_filename = NULL,
     .rendezvous_filename = NULL,
     .wait_to_connect = 4,
@@ -297,6 +299,10 @@ pmix_status_t component_close(void)
         unlink(mca_ptl_tcp_component.nspace_filename);
         free(mca_ptl_tcp_component.nspace_filename);
     }
+    if (NULL != mca_ptl_tcp_component.pid_filename) {
+        unlink(mca_ptl_tcp_component.pid_filename);
+        free(mca_ptl_tcp_component.pid_filename);
+    }
     if (NULL != mca_ptl_tcp_component.rendezvous_filename) {
         unlink(mca_ptl_tcp_component.rendezvous_filename);
         free(mca_ptl_tcp_component.rendezvous_filename);
@@ -354,8 +360,8 @@ static pmix_status_t setup_listener(pmix_info_t info[], size_t ninfo,
     bool session_tool = false;
     bool system_tool = false;
     pmix_socklen_t addrlen;
-    char *prefix, myhost[PMIX_MAXHOSTNAMELEN];
-    char myconnhost[PMIX_MAXHOSTNAMELEN];
+    char *prefix, myhost[PMIX_MAXHOSTNAMELEN] = {0};
+    char myconnhost[PMIX_MAXHOSTNAMELEN] = {0};
     int myport;
     pmix_kval_t *urikv;
 
@@ -618,17 +624,17 @@ static pmix_status_t setup_listener(pmix_info_t info[], size_t ninfo,
         goto sockerror;
     }
 
-    gethostname(myhost, sizeof(myhost));
+    gethostname(myhost, sizeof(myhost)-1);
     if (AF_INET == mca_ptl_tcp_component.connection.ss_family) {
         prefix = "tcp4://";
         myport = ntohs(((struct sockaddr_in*) &mca_ptl_tcp_component.connection)->sin_port);
         inet_ntop(AF_INET, &((struct sockaddr_in*) &mca_ptl_tcp_component.connection)->sin_addr,
-                  myconnhost, PMIX_MAXHOSTNAMELEN);
+                  myconnhost, PMIX_MAXHOSTNAMELEN-1);
     } else if (AF_INET6 == mca_ptl_tcp_component.connection.ss_family) {
         prefix = "tcp6://";
         myport = ntohs(((struct sockaddr_in6*) &mca_ptl_tcp_component.connection)->sin6_port);
         inet_ntop(AF_INET6, &((struct sockaddr_in6*) &mca_ptl_tcp_component.connection)->sin6_addr,
-                  myconnhost, PMIX_MAXHOSTNAMELEN);
+                  myconnhost, PMIX_MAXHOSTNAMELEN-1);
     } else {
         goto sockerror;
     }
@@ -750,10 +756,10 @@ static pmix_status_t setup_listener(pmix_info_t info[], size_t ninfo,
         FILE *fp;
         pid_t mypid;
 
-        /* first output to a file based on pid */
+        /* first output to a std file */
         mypid = getpid();
-        if (0 > asprintf(&mca_ptl_tcp_component.session_filename, "%s/pmix.%s.tool.%d",
-                         mca_ptl_tcp_component.session_tmpdir, myhost, mypid)) {
+        if (0 > asprintf(&mca_ptl_tcp_component.session_filename, "%s/pmix.%s.tool",
+                         mca_ptl_tcp_component.session_tmpdir, myhost)) {
             CLOSE_THE_SOCKET(lt->socket);
             goto sockerror;
         }
@@ -784,6 +790,40 @@ static pmix_status_t setup_listener(pmix_info_t info[], size_t ninfo,
             goto sockerror;
         }
 
+        /* now output to a file based on pid */
+        mypid = getpid();
+        if (0 > asprintf(&mca_ptl_tcp_component.pid_filename, "%s/pmix.%s.tool.%d",
+                         mca_ptl_tcp_component.session_tmpdir, myhost, mypid)) {
+            CLOSE_THE_SOCKET(lt->socket);
+            goto sockerror;
+        }
+        pmix_output_verbose(2, pmix_ptl_base_framework.framework_output,
+                            "WRITING TOOL FILE %s",
+                            mca_ptl_tcp_component.pid_filename);
+        fp = fopen(mca_ptl_tcp_component.pid_filename, "w");
+        if (NULL == fp) {
+            pmix_output(0, "Impossible to open the file %s in write mode\n", mca_ptl_tcp_component.pid_filename);
+            PMIX_ERROR_LOG(PMIX_ERR_FILE_OPEN_FAILURE);
+            CLOSE_THE_SOCKET(lt->socket);
+            free(mca_ptl_tcp_component.pid_filename);
+            mca_ptl_tcp_component.pid_filename = NULL;
+            goto sockerror;
+        }
+
+        /* output my URI */
+        fprintf(fp, "%s\n", lt->uri);
+        /* add a flag that indicates we accept v2.1 protocols */
+        fprintf(fp, "%s\n", PMIX_VERSION);
+        fclose(fp);
+        /* set the file mode */
+        if (0 != chmod(mca_ptl_tcp_component.pid_filename, S_IRUSR | S_IWUSR | S_IRGRP)) {
+            PMIX_ERROR_LOG(PMIX_ERR_FILE_OPEN_FAILURE);
+            CLOSE_THE_SOCKET(lt->socket);
+            free(mca_ptl_tcp_component.pid_filename);
+            mca_ptl_tcp_component.pid_filename = NULL;
+            goto sockerror;
+        }
+
         /* now output it into a file based on my nspace */
 
         if (0 > asprintf(&mca_ptl_tcp_component.nspace_filename, "%s/pmix.%s.tool.%s",
@@ -858,7 +898,7 @@ static char **split_and_resolve(char **orig_str, char *name)
 {
     int i, ret, save, if_index;
     char **argv, *str, *tmp;
-    char if_name[IF_NAMESIZE];
+    char if_name[PMIX_IF_NAMESIZE];
     struct sockaddr_storage argv_inaddr, if_inaddr;
     uint32_t argv_prefix;
 
@@ -957,7 +997,7 @@ static void connection_handler(int sd, short args, void *cbdata)
     pmix_ptl_hdr_t hdr;
     pmix_peer_t *peer;
     pmix_rank_t rank=0;
-    pmix_status_t rc;
+    pmix_status_t rc, reply;
     char *msg, *mg, *version;
     char *sec, *bfrops, *gds;
     pmix_bfrop_buffer_type_t bftype;
@@ -1344,18 +1384,28 @@ static void connection_handler(int sd, short args, void *cbdata)
              * of local clients. So let's start by searching for
              * the nspace object */
             nptr = NULL;
-            PMIX_LIST_FOREACH(tmp, &pmix_server_globals.nspaces, pmix_namespace_t) {
+            PMIX_LIST_FOREACH(tmp, &pmix_globals.nspaces, pmix_namespace_t) {
                 if (0 == strcmp(tmp->nspace, nspace)) {
                     nptr = tmp;
                     break;
                 }
             }
             if (NULL == nptr) {
-                /* we don't know this namespace, reject it */
-                free(msg);
-                /* send an error reply to the client */
-                rc = PMIX_ERR_NOT_FOUND;
-                goto error;
+                /* it is possible that this is a tool inside of
+                 * a job-script as part of a multi-spawn operation.
+                 * Since each tool invocation may have finalized and
+                 * terminated, the tool will appear to "terminate", thus
+                 * causing us to cleanup all references to it, and then
+                 * reappear. So we don't reject this connection request.
+                 * Instead, we create the nspace and rank objects for
+                 * it and let the RM/host decide if this behavior
+                 * is allowed */
+                nptr = PMIX_NEW(pmix_namespace_t);
+                if (NULL == nptr) {
+                    rc = PMIX_ERR_NOMEM;
+                    goto error;
+                }
+                nptr->nspace = strdup(nspace);
             }
             /* now look for the rank */
             info = NULL;
@@ -1367,11 +1417,13 @@ static void connection_handler(int sd, short args, void *cbdata)
                 }
             }
             if (!found) {
-                /* rank unknown, reject it */
-                free(msg);
-                /* send an error reply to the client */
-                rc = PMIX_ERR_NOT_FOUND;
-                goto error;
+                /* see above note about not finding nspace */
+                info = PMIX_NEW(pmix_rank_info_t);
+                info->pname.nspace = strdup(nspace);
+                info->pname.rank = rank;
+                info->uid = pnd->uid;
+                info->gid = pnd->gid;
+                pmix_list_append(&nptr->ranks, &info->super);
             }
             PMIX_RETAIN(info);
             peer->info = info;
@@ -1482,7 +1534,7 @@ static void connection_handler(int sd, short args, void *cbdata)
 
     /* see if we know this nspace */
     nptr = NULL;
-    PMIX_LIST_FOREACH(tmp, &pmix_server_globals.nspaces, pmix_namespace_t) {
+    PMIX_LIST_FOREACH(tmp, &pmix_globals.nspaces, pmix_namespace_t) {
         if (0 == strcmp(tmp->nspace, nspace)) {
             nptr = tmp;
             break;
@@ -1610,22 +1662,13 @@ static void connection_handler(int sd, short args, void *cbdata)
     /* validate the connection */
     cred.bytes = pnd->cred;
     cred.size = pnd->len;
-    PMIX_PSEC_VALIDATE_CONNECTION(rc, peer, NULL, 0, NULL, NULL, &cred);
-    if (PMIX_SUCCESS != rc) {
-        pmix_output_verbose(2, pmix_ptl_base_framework.framework_output,
-                            "validation of client connection failed");
-        info->proc_cnt--;
-        pmix_pointer_array_set_item(&pmix_server_globals.clients, peer->index, NULL);
-        PMIX_RELEASE(peer);
-        /* send an error reply to the client */
-        goto error;
-    }
+    PMIX_PSEC_VALIDATE_CONNECTION(reply, peer, NULL, 0, NULL, NULL, &cred);
 
     pmix_output_verbose(2, pmix_ptl_base_framework.framework_output,
-                        "client connection validated");
+                        "client connection validated with status=%d", reply);
 
     /* tell the client all is good */
-    u32 = htonl(PMIX_SUCCESS);
+    u32 = htonl(reply);
     if (PMIX_SUCCESS != (rc = pmix_ptl_base_send_blocking(pnd->sd, (char*)&u32, sizeof(uint32_t)))) {
         PMIX_ERROR_LOG(rc);
         info->proc_cnt--;
@@ -1635,6 +1678,22 @@ static void connection_handler(int sd, short args, void *cbdata)
         PMIX_RELEASE(pnd);
         return;
     }
+    /* If needed perform the handshake. The macro will update reply */
+    PMIX_PSEC_SERVER_HANDSHAKE_IFNEED(reply, peer, NULL, 0, NULL, NULL, &cred);
+
+    /* It is possible that connection validation failed
+     * We need to reply to the client first and cleanup after */
+    if (PMIX_SUCCESS != reply) {
+        pmix_output_verbose(2, pmix_ptl_base_framework.framework_output,
+                            "validation of client connection failed");
+        info->proc_cnt--;
+        pmix_pointer_array_set_item(&pmix_server_globals.clients, peer->index, NULL);
+        PMIX_RELEASE(peer);
+        /* send an error reply to the client */
+        goto error;
+    }
+
+
       /* send the client's array index */
     u32 = htonl(peer->index);
       if (PMIX_SUCCESS != (rc = pmix_ptl_base_send_blocking(pnd->sd, (char*)&u32, sizeof(uint32_t)))) {
@@ -1697,7 +1756,7 @@ static void process_cbfunc(int sd, short args, void *cbdata)
     pmix_namespace_t *nptr;
     pmix_rank_info_t *info;
     pmix_peer_t *peer;
-    int rc;
+    pmix_status_t rc, reply;
     uint32_t u32;
     pmix_info_t ginfo;
     pmix_byte_object_t cred;
@@ -1779,7 +1838,7 @@ static void process_cbfunc(int sd, short args, void *cbdata)
     if (5 != pnd->flag && 8 != pnd->flag) {
         PMIX_RETAIN(nptr);
         nptr->nspace = strdup(cd->proc.nspace);
-        pmix_list_append(&pmix_server_globals.nspaces, &nptr->super);
+        pmix_list_append(&pmix_globals.nspaces, &nptr->super);
         info = PMIX_NEW(pmix_rank_info_t);
         info->pname.nspace = strdup(nptr->nspace);
         info->pname.rank = cd->proc.rank;
@@ -1807,7 +1866,7 @@ static void process_cbfunc(int sd, short args, void *cbdata)
     peer->nptr->compat.psec = pmix_psec_base_assign_module(pnd->psec);
     if (NULL == peer->nptr->compat.psec) {
         PMIX_RELEASE(peer);
-        pmix_list_remove_item(&pmix_server_globals.nspaces, &nptr->super);
+        pmix_list_remove_item(&pmix_globals.nspaces, &nptr->super);
         PMIX_RELEASE(nptr);  // will release the info object
         CLOSE_THE_SOCKET(pnd->sd);
         goto done;
@@ -1822,7 +1881,7 @@ static void process_cbfunc(int sd, short args, void *cbdata)
     PMIX_INFO_DESTRUCT(&ginfo);
     if (NULL == peer->nptr->compat.gds) {
         PMIX_RELEASE(peer);
-        pmix_list_remove_item(&pmix_server_globals.nspaces, &nptr->super);
+        pmix_list_remove_item(&pmix_globals.nspaces, &nptr->super);
         PMIX_RELEASE(nptr);  // will release the info object
         CLOSE_THE_SOCKET(pnd->sd);
         goto done;
@@ -1841,7 +1900,7 @@ static void process_cbfunc(int sd, short args, void *cbdata)
     req = PMIX_NEW(pmix_iof_req_t);
     if (NULL == req) {
         PMIX_RELEASE(peer);
-        pmix_list_remove_item(&pmix_server_globals.nspaces, &nptr->super);
+        pmix_list_remove_item(&pmix_globals.nspaces, &nptr->super);
         PMIX_RELEASE(nptr);  // will release the info object
         CLOSE_THE_SOCKET(pnd->sd);
         goto done;
@@ -1856,13 +1915,28 @@ static void process_cbfunc(int sd, short args, void *cbdata)
     /* validate the connection */
     cred.bytes = pnd->cred;
     cred.size = pnd->len;
-    PMIX_PSEC_VALIDATE_CONNECTION(rc, peer, NULL, 0, NULL, NULL, &cred);
-    if (PMIX_SUCCESS != rc) {
+    PMIX_PSEC_VALIDATE_CONNECTION(reply, peer, NULL, 0, NULL, NULL, &cred);
+    /* communicate the result to the other side */
+    u32 = htonl(reply);
+    if (PMIX_SUCCESS != (rc = pmix_ptl_base_send_blocking(pnd->sd, (char*)&u32, sizeof(uint32_t)))) {
+        PMIX_ERROR_LOG(rc);
+        PMIX_RELEASE(peer);
+        pmix_list_remove_item(&pmix_server_globals.nspaces, &nptr->super);
+        PMIX_RELEASE(nptr);  // will release the info object
+        CLOSE_THE_SOCKET(pnd->sd);
+        goto done;
+    }
+
+    /* If needed perform the handshake. The macro will update reply */
+    PMIX_PSEC_SERVER_HANDSHAKE_IFNEED(reply, peer, NULL, 0, NULL, NULL, &cred);
+
+    /* If verification wasn't successful - stop here */
+    if (PMIX_SUCCESS != reply) {
         pmix_output_verbose(2, pmix_ptl_base_framework.framework_output,
                             "validation of tool credentials failed: %s",
                             PMIx_Error_string(rc));
         PMIX_RELEASE(peer);
-        pmix_list_remove_item(&pmix_server_globals.nspaces, &nptr->super);
+        pmix_list_remove_item(&pmix_globals.nspaces, &nptr->super);
         PMIX_RELEASE(nptr);  // will release the info object
         CLOSE_THE_SOCKET(pnd->sd);
         goto done;
@@ -1875,12 +1949,12 @@ static void process_cbfunc(int sd, short args, void *cbdata)
         PMIX_RELEASE(pnd);
         PMIX_RELEASE(cd);
         PMIX_RELEASE(peer);
-        pmix_list_remove_item(&pmix_server_globals.nspaces, &nptr->super);
+        pmix_list_remove_item(&pmix_globals.nspaces, &nptr->super);
         PMIX_RELEASE(nptr);  // will release the info object
         /* probably cannot send an error reply if we are out of memory */
         return;
     }
-    info->peerid = peer->index;
+    peer->info->peerid = peer->index;
 
     /* start the events for this tool */
     pmix_event_assign(&peer->recv_event, pmix_globals.evbase, peer->sd,
@@ -1906,8 +1980,8 @@ static void cnct_cbfunc(pmix_status_t status,
     pmix_setup_caddy_t *cd;
 
     pmix_output_verbose(2, pmix_ptl_base_framework.framework_output,
-                        "pmix:tcp:cnct_cbfunc returning %s:%d",
-                        proc->nspace, proc->rank);
+                        "pmix:tcp:cnct_cbfunc returning %s:%d %s",
+                        proc->nspace, proc->rank, PMIx_Error_string(status));
 
     /* need to thread-shift this into our context */
     cd = PMIX_NEW(pmix_setup_caddy_t);
