@@ -15,7 +15,7 @@
  * Copyright (c) 2009      Oak Ridge National Labs.  All rights reserved.
  * Copyright (c) 2010-2015 Los Alamos National Security, LLC.
  *                         All rights reserved.
- * Copyright (c) 2013-2018 Intel, Inc. All rights reserved.
+ * Copyright (c) 2013-2019 Intel, Inc.  All rights reserved.
  * Copyright (c) 2015      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
@@ -33,7 +33,9 @@
 #include <unistd.h>
 #endif
 #include PMIX_EVENT_HEADER
-#include "event2/thread.h"
+#if ! PMIX_HAVE_LIBEV
+#include PMIX_EVENT2_THREAD_HEADER
+#endif
 
 #include <pmix_rename.h>
 
@@ -70,6 +72,8 @@ PMIX_EXPORT bool pmix_init_called = false;
 PMIX_EXPORT pmix_globals_t pmix_globals = {
     .init_cntr = 0,
     .mypeer = NULL,
+    .hostname = NULL,
+    .nodeid = UINT32_MAX,
     .pindex = 0,
     .evbase = NULL,
     .external_evbase = false,
@@ -80,6 +84,15 @@ PMIX_EXPORT pmix_globals_t pmix_globals = {
 };
 
 
+static void _notification_eviction_cbfunc(struct pmix_hotel_t *hotel,
+                                          int room_num,
+                                          void *occupant)
+{
+    pmix_notify_caddy_t *cache = (pmix_notify_caddy_t*)occupant;
+    PMIX_RELEASE(cache);
+}
+
+
 int pmix_rte_init(pmix_proc_type_t type,
                   pmix_info_t info[], size_t ninfo,
                   pmix_ptl_cbfunc_t cbfunc)
@@ -87,6 +100,7 @@ int pmix_rte_init(pmix_proc_type_t type,
     int ret, debug_level;
     char *error = NULL, *evar;
     size_t n;
+    char hostname[PMIX_MAXHOSTNAMELEN] = {0};
 
     if( ++pmix_initialized != 1 ) {
         if( pmix_initialized < 1 ) {
@@ -147,6 +161,8 @@ int pmix_rte_init(pmix_proc_type_t type,
     }
 
     /* setup the globals structure */
+    gethostname(hostname, PMIX_MAXHOSTNAMELEN-1);
+    pmix_globals.hostname = strdup(hostname);
     memset(&pmix_globals.myid.nspace, 0, PMIX_MAX_NSLEN+1);
     pmix_globals.myid.rank = PMIX_RANK_INVALID;
     PMIX_CONSTRUCT(&pmix_globals.events, pmix_events_t);
@@ -154,8 +170,17 @@ int pmix_rte_init(pmix_proc_type_t type,
     pmix_globals.event_window.tv_usec = 0;
     PMIX_CONSTRUCT(&pmix_globals.cached_events, pmix_list_t);
     /* construct the global notification ring buffer */
-    PMIX_CONSTRUCT(&pmix_globals.notifications, pmix_ring_buffer_t);
-    pmix_ring_buffer_init(&pmix_globals.notifications, 256);
+    PMIX_CONSTRUCT(&pmix_globals.notifications, pmix_hotel_t);
+    ret = pmix_hotel_init(&pmix_globals.notifications, pmix_globals.max_events,
+                          pmix_globals.evbase, pmix_globals.event_eviction_time,
+                          _notification_eviction_cbfunc);
+    PMIX_CONSTRUCT(&pmix_globals.nspaces, pmix_list_t);
+
+    if (PMIX_SUCCESS != ret) {
+        error = "notification hotel init";
+        goto return_error;
+    }
+
     /* and setup the iof request tracking list */
     PMIX_CONSTRUCT(&pmix_globals.iof_requests, pmix_list_t);
 
@@ -223,7 +248,7 @@ int pmix_rte_init(pmix_proc_type_t type,
     pmix_globals.mypeer->proc_type = type | PMIX_PROC_V3;
     /* create an nspace object for ourselves - we will
      * fill in the nspace name later */
-    pmix_globals.mypeer->nptr = PMIX_NEW(pmix_nspace_t);
+    pmix_globals.mypeer->nptr = PMIX_NEW(pmix_namespace_t);
     if (NULL == pmix_globals.mypeer->nptr) {
         PMIX_RELEASE(pmix_globals.mypeer);
         ret = PMIX_ERR_NOMEM;
@@ -233,9 +258,19 @@ int pmix_rte_init(pmix_proc_type_t type,
     /* scan incoming info for directives */
     if (NULL != info) {
         for (n=0; n < ninfo; n++) {
-            if (0 == strcmp(PMIX_EVENT_BASE, info[n].key)) {
+            if (PMIX_CHECK_KEY(&info[n], PMIX_EVENT_BASE)) {
                 pmix_globals.evbase = (pmix_event_base_t*)info[n].value.data.ptr;
                 pmix_globals.external_evbase = true;
+            } else if (PMIX_CHECK_KEY(&info[n], PMIX_HOSTNAME)) {
+                if (NULL != pmix_globals.hostname) {
+                    free(pmix_globals.hostname);
+                }
+                pmix_globals.hostname = strdup(info[n].value.data.string);
+            } else if (PMIX_CHECK_KEY(&info[n], PMIX_NODEID)) {
+                PMIX_VALUE_GET_NUMBER(ret, &info[n].value, pmix_globals.nodeid, uint32_t);
+                if (PMIX_SUCCESS != ret) {
+                    goto return_error;
+                }
             }
         }
     }
