@@ -1,8 +1,8 @@
 /* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
- * Copyright (c) 2014-2018 Intel, Inc. All rights reserved.
- * Copyright (c) 2014-2015 Research Organization for Information Science
- *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2014-2019 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2014-2019 Research Organization for Information Science
+ *                         and Technology (RIST).  All rights reserved.
  * Copyright (c) 2014-2015 Artem Y. Polyakov <artpol84@gmail.com>.
  *                         All rights reserved.
  * Copyright (c) 2016      Mellanox Technologies, Inc.
@@ -51,6 +51,7 @@
 #include "src/mca/gds/gds.h"
 #include "src/util/argv.h"
 #include "src/util/error.h"
+#include "src/util/name_fns.h"
 #include "src/util/output.h"
 #include "src/util/pmix_environ.h"
 
@@ -126,6 +127,7 @@ pmix_status_t pmix_server_get(pmix_buffer_t *buf,
     pmix_dmdx_request_t *req;
     bool local;
     bool localonly = false;
+    bool diffnspace = false;
     struct timeval tv = {0, 0};
     pmix_buffer_t pbkt, pkt;
     pmix_byte_object_t bo;
@@ -133,10 +135,10 @@ pmix_status_t pmix_server_get(pmix_buffer_t *buf,
     pmix_proc_t proc;
     char *data;
     size_t sz, n;
-    pmix_peer_t *peer;
 
     pmix_output_verbose(2, pmix_server_globals.get_output,
-                        "recvd GET");
+                        "%s recvd GET",
+                        PMIX_NAME_PRINT(&pmix_globals.myid));
 
     /* setup */
     memset(nspace, 0, sizeof(nspace));
@@ -191,13 +193,19 @@ pmix_status_t pmix_server_get(pmix_buffer_t *buf,
 
     /* find the nspace object for this client */
     nptr = NULL;
-    PMIX_LIST_FOREACH(ns, &pmix_server_globals.nspaces, pmix_namespace_t) {
+    PMIX_LIST_FOREACH(ns, &pmix_globals.nspaces, pmix_namespace_t) {
         if (0 == strcmp(nspace, ns->nspace)) {
             nptr = ns;
             break;
         }
     }
 
+    /* check if the nspace of the requestor is different from
+     * the nspace of the target process */
+    if (!PMIX_CHECK_NSPACE(nspace, cd->peer->info->pname.nspace)) {
+        diffnspace = true;
+    }
+
     pmix_output_verbose(2, pmix_server_globals.get_output,
                         "%s:%d EXECUTE GET FOR %s:%d ON BEHALF OF %s:%d",
                         pmix_globals.myid.nspace,
@@ -294,10 +302,10 @@ pmix_status_t pmix_server_get(pmix_buffer_t *buf,
         return PMIX_SUCCESS;
     }
 
-    /* this nspace is known, so we can process the request.
-     * if the rank is wildcard, then they are asking for the
-     * job-level info for this nspace - provide it */
-    if (PMIX_RANK_WILDCARD == rank) {
+    /* the target nspace is known, so we can process the request.
+     * if the rank is wildcard, or the nspace is different, then
+     * they are asking for the job-level info for this nspace - provide it */
+    if (PMIX_RANK_WILDCARD == rank || diffnspace) {
         /* see if we have the job-level info - we won't have it
          * if we have no local procs and haven't already asked
          * for it, so there is no guarantee we have it */
@@ -309,21 +317,32 @@ pmix_status_t pmix_server_get(pmix_buffer_t *buf,
          * can retrieve the info from that GDS. Otherwise,
          * we need to retrieve it from our own */
         PMIX_CONSTRUCT(&cb, pmix_cb_t);
-        peer = pmix_globals.mypeer;
         /* this data is for a local client, so give the gds the
          * option of returning a complete copy of the data,
          * or returning a pointer to local storage */
         cb.proc = &proc;
         cb.scope = PMIX_SCOPE_UNDEF;
         cb.copy = false;
-        PMIX_GDS_FETCH_KV(rc, peer, &cb);
+        PMIX_GDS_FETCH_KV(rc, pmix_globals.mypeer, &cb);
         if (PMIX_SUCCESS != rc) {
             PMIX_DESTRUCT(&cb);
             return rc;
         }
+        /* if the requested rank is not WILDCARD, then retrieve the
+         * job-specific data for that rank - a scope of UNDEF
+         * will direct the GDS to provide it. Anything found will
+         * simply be added to the cb.kvs list */
+        if (PMIX_RANK_WILDCARD != rank) {
+            proc.rank = rank;
+            PMIX_GDS_FETCH_KV(rc, pmix_globals.mypeer, &cb);
+            if (PMIX_SUCCESS != rc) {
+                PMIX_DESTRUCT(&cb);
+                return rc;
+            }
+        }
         PMIX_CONSTRUCT(&pkt, pmix_buffer_t);
         /* assemble the provided data into a byte object */
-        PMIX_GDS_ASSEMB_KVS_REQ(rc, peer, &proc, &cb.kvs, &pkt, cd);
+        PMIX_GDS_ASSEMB_KVS_REQ(rc, pmix_globals.mypeer, &proc, &cb.kvs, &pkt, cd);
         if (PMIX_SUCCESS != rc) {
             PMIX_ERROR_LOG(rc);
             PMIX_DESTRUCT(&cb);
@@ -333,7 +352,7 @@ pmix_status_t pmix_server_get(pmix_buffer_t *buf,
         PMIX_DESTRUCT(&pkt);
         /* pack it into the payload */
         PMIX_CONSTRUCT(&pbkt, pmix_buffer_t);
-        PMIX_BFROPS_PACK(rc, cd->peer, &pbkt, &bo, 1, PMIX_BYTE_OBJECT);
+        PMIX_BFROPS_PACK(rc, pmix_globals.mypeer, &pbkt, &bo, 1, PMIX_BYTE_OBJECT);
         free(bo.bytes);
         if (PMIX_SUCCESS != rc) {
             PMIX_ERROR_LOG(rc);
@@ -604,6 +623,7 @@ static pmix_status_t _satisfy_request(pmix_namespace_t *nptr, pmix_rank_t rank,
     char *data = NULL;
     size_t sz = 0;
     pmix_scope_t scope = PMIX_SCOPE_UNDEF;
+    bool diffnspace = false;
 
     pmix_output_verbose(2, pmix_server_globals.get_output,
                         "%s:%d SATISFY REQUEST CALLED",
@@ -617,10 +637,18 @@ static pmix_status_t _satisfy_request(pmix_namespace_t *nptr, pmix_rank_t rank,
     PMIX_CONSTRUCT(&pbkt, pmix_buffer_t);
     pmix_strncpy(proc.nspace, nptr->nspace, PMIX_MAX_NSLEN);
 
-    /* if we have local clients of this nspace, then we use
-     * the corresponding GDS to retrieve the data. Otherwise,
-     * the data will have been stored under our GDS */
-    if (0 < nptr->nlocalprocs) {
+    if (!PMIX_CHECK_NSPACE(nptr->nspace, cd->peer->info->pname.nspace)) {
+        diffnspace = true;
+    }
+
+    /* if rank is PMIX_RANK_UNDEF, then it was stored in our GDS */
+    if (PMIX_RANK_UNDEF == rank) {
+        scope = PMIX_GLOBAL;  // we have to search everywhere
+        peer = pmix_globals.mypeer;
+    } else if (0 < nptr->nlocalprocs) {
+        /* if we have local clients of this nspace, then we use
+         * the corresponding GDS to retrieve the data. Otherwise,
+         * the data will have been stored under our GDS */
         if (local) {
             *local = true;
         }
@@ -660,8 +688,7 @@ static pmix_status_t _satisfy_request(pmix_namespace_t *nptr, pmix_rank_t rank,
     /* if they are asking about a rank from an nspace different
      * from their own, or they gave a rank of "wildcard", then
      * include a copy of the job-level info */
-    if (PMIX_RANK_WILDCARD == rank ||
-        0 != strncmp(nptr->nspace, cd->peer->info->pname.nspace, PMIX_MAX_NSLEN)) {
+    if (PMIX_RANK_WILDCARD == rank || diffnspace) {
         proc.rank = PMIX_RANK_WILDCARD;
         PMIX_CONSTRUCT(&cb, pmix_cb_t);
         /* this data is requested by a local client, so give the gds the option
@@ -674,7 +701,7 @@ static pmix_status_t _satisfy_request(pmix_namespace_t *nptr, pmix_rank_t rank,
         if (PMIX_SUCCESS == rc) {
             PMIX_CONSTRUCT(&pkt, pmix_buffer_t);
             /* assemble the provided data into a byte object */
-            PMIX_GDS_ASSEMB_KVS_REQ(rc, cd->peer, &proc, &cb.kvs, &pkt, cd);
+            PMIX_GDS_ASSEMB_KVS_REQ(rc, pmix_globals.mypeer, &proc, &cb.kvs, &pkt, cd);
             if (rc != PMIX_SUCCESS) {
                 PMIX_ERROR_LOG(rc);
                 PMIX_DESTRUCT(&pkt);
@@ -720,7 +747,8 @@ static pmix_status_t _satisfy_request(pmix_namespace_t *nptr, pmix_rank_t rank,
 
     /* retrieve the data for the specific rank they are asking about */
     if (PMIX_RANK_WILDCARD != rank) {
-        if (!PMIX_PROC_IS_SERVER(peer) && !peer->commit_cnt) {
+        if (!PMIX_PROC_IS_SERVER(peer) && 0 == peer->commit_cnt) {
+            PMIX_ERROR_LOG(PMIX_ERR_NOT_FOUND);
             /* this condition works only for local requests, server does
              * count commits for local ranks, and check this count when
              * local request.
@@ -743,7 +771,11 @@ static pmix_status_t _satisfy_request(pmix_namespace_t *nptr, pmix_rank_t rank,
             found = true;
             PMIX_CONSTRUCT(&pkt, pmix_buffer_t);
             /* assemble the provided data into a byte object */
-            PMIX_GDS_ASSEMB_KVS_REQ(rc, cd->peer, &proc, &cb.kvs, &pkt, cd);
+            if (PMIX_RANK_UNDEF == rank || diffnspace) {
+                PMIX_GDS_ASSEMB_KVS_REQ(rc, pmix_globals.mypeer, &proc, &cb.kvs, &pkt, cd);
+            } else {
+                PMIX_GDS_ASSEMB_KVS_REQ(rc, cd->peer, &proc, &cb.kvs, &pkt, cd);
+            }
             if (rc != PMIX_SUCCESS) {
                 PMIX_ERROR_LOG(rc);
                 PMIX_DESTRUCT(&pkt);
@@ -789,6 +821,7 @@ static pmix_status_t _satisfy_request(pmix_namespace_t *nptr, pmix_rank_t rank,
         }
         PMIX_DESTRUCT(&cb);
     }
+
     PMIX_UNLOAD_BUFFER(&pbkt, data, sz);
     PMIX_DESTRUCT(&pbkt);
 
@@ -896,7 +929,7 @@ static void _process_dmdx_reply(int fd, short args, void *cbdata)
 
     /* find the nspace object for the proc whose data is being received */
     nptr = NULL;
-    PMIX_LIST_FOREACH(ns, &pmix_server_globals.nspaces, pmix_namespace_t) {
+    PMIX_LIST_FOREACH(ns, &pmix_globals.nspaces, pmix_namespace_t) {
         if (0 == strcmp(caddy->lcd->proc.nspace, ns->nspace)) {
             nptr = ns;
             break;
@@ -910,7 +943,7 @@ static void _process_dmdx_reply(int fd, short args, void *cbdata)
         nptr = PMIX_NEW(pmix_namespace_t);
         nptr->nspace = strdup(caddy->lcd->proc.nspace);
         /* add to the list */
-        pmix_list_append(&pmix_server_globals.nspaces, &nptr->super);
+        pmix_list_append(&pmix_globals.nspaces, &nptr->super);
     }
 
     /* if the request was successfully satisfied, then store the data.
