@@ -1,5 +1,5 @@
  /*
- * Copyright (c) 2015-2018 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2015-2019 Intel, Inc.  All rights reserved.
  * Copyright (c) 2015-2018 Mellanox Technologies, Inc.
  *                         All rights reserved.
  * Copyright (c) 2016      Research Organization for Information Science
@@ -77,7 +77,7 @@ PMIX_CLASS_INSTANCE(server_nspace_t,
                     nscon, nsdes);
 
 static int server_send_procs(void);
-static void server_read_cb(evutil_socket_t fd, short event, void *arg);
+static void server_read_cb(int fd, short event, void *arg);
 static int srv_wait_all(double timeout);
 static int server_fwd_msg(msg_hdr_t *msg_hdr, char *buf, size_t size);
 static int server_send_msg(msg_hdr_t *msg_hdr, char *data, size_t size);
@@ -93,56 +93,6 @@ static void release_cb(pmix_status_t status, void *cbdata)
     *ptr = 0;
 }
 
-static void set_namespace(int local_size, int univ_size, char *ranks, char *name)
-{
-    size_t ninfo;
-    pmix_info_t *info;
-    ninfo = 8;
-    char *regex, *ppn;
-
-    PMIX_INFO_CREATE(info, ninfo);
-    pmix_strncpy(info[0].key, PMIX_UNIV_SIZE, PMIX_MAX_KEYLEN);
-    info[0].value.type = PMIX_UINT32;
-    info[0].value.data.uint32 = univ_size;
-
-    pmix_strncpy(info[1].key, PMIX_SPAWNED, PMIX_MAX_KEYLEN);
-    info[1].value.type = PMIX_UINT32;
-    info[1].value.data.uint32 = 0;
-
-    pmix_strncpy(info[2].key, PMIX_LOCAL_SIZE, PMIX_MAX_KEYLEN);
-    info[2].value.type = PMIX_UINT32;
-    info[2].value.data.uint32 = local_size;
-
-    pmix_strncpy(info[3].key, PMIX_LOCAL_PEERS, PMIX_MAX_KEYLEN);
-    info[3].value.type = PMIX_STRING;
-    info[3].value.data.string = strdup(ranks);
-
-    PMIx_generate_regex(NODE_NAME, &regex);
-    pmix_strncpy(info[4].key, PMIX_NODE_MAP, PMIX_MAX_KEYLEN);
-    info[4].value.type = PMIX_STRING;
-    info[4].value.data.string = regex;
-
-    PMIx_generate_ppn(ranks, &ppn);
-    pmix_strncpy(info[5].key, PMIX_PROC_MAP, PMIX_MAX_KEYLEN);
-    info[5].value.type = PMIX_STRING;
-    info[5].value.data.string = ppn;
-
-    pmix_strncpy(info[6].key, PMIX_JOB_SIZE, PMIX_MAX_KEYLEN);
-    info[6].value.type = PMIX_UINT32;
-    info[6].value.data.uint32 = univ_size;
-
-    pmix_strncpy(info[7].key, PMIX_APPNUM, PMIX_MAX_KEYLEN);
-    info[7].value.type = PMIX_UINT32;
-    info[7].value.data.uint32 = getpid ();
-
-    int in_progress = 1, rc;
-    if (PMIX_SUCCESS == (rc = PMIx_server_register_nspace(name, local_size,
-                                    info, ninfo, release_cb, &in_progress))) {
-        PMIX_WAIT_FOR_COMPLETION(in_progress);
-    }
-    PMIX_INFO_FREE(info, ninfo);
-}
-
 static void fill_seq_ranks_array(size_t nprocs, int base_rank, char **ranks)
 {
     uint32_t i;
@@ -165,6 +115,70 @@ static void fill_seq_ranks_array(size_t nprocs, int base_rank, char **ranks)
     }
 }
 
+static void set_namespace(int local_size, int univ_size,
+                          int base_rank, char *name)
+{
+    size_t ninfo;
+    pmix_info_t *info;
+    ninfo = 8;
+    char *regex, *ppn;
+    char *ranks = NULL;
+
+    PMIX_INFO_CREATE(info, ninfo);
+    pmix_strncpy(info[0].key, PMIX_UNIV_SIZE, PMIX_MAX_KEYLEN);
+    info[0].value.type = PMIX_UINT32;
+    info[0].value.data.uint32 = univ_size;
+
+    pmix_strncpy(info[1].key, PMIX_SPAWNED, PMIX_MAX_KEYLEN);
+    info[1].value.type = PMIX_UINT32;
+    info[1].value.data.uint32 = 0;
+
+    pmix_strncpy(info[2].key, PMIX_LOCAL_SIZE, PMIX_MAX_KEYLEN);
+    info[2].value.type = PMIX_UINT32;
+    info[2].value.data.uint32 = local_size;
+
+    /* generate the array of local peers */
+    fill_seq_ranks_array(local_size, base_rank, &ranks);
+    if (NULL == ranks) {
+        return;
+    }
+    pmix_strncpy(info[3].key, PMIX_LOCAL_PEERS, PMIX_MAX_KEYLEN);
+    info[3].value.type = PMIX_STRING;
+    info[3].value.data.string = strdup(ranks);
+    free(ranks);
+
+    PMIx_generate_regex(NODE_NAME, &regex);
+    pmix_strncpy(info[4].key, PMIX_NODE_MAP, PMIX_MAX_KEYLEN);
+    info[4].value.type = PMIX_STRING;
+    info[4].value.data.string = strdup(regex);
+
+    /* generate the global proc map */
+    fill_seq_ranks_array(univ_size, 0, &ranks);
+    if (NULL == ranks) {
+        return;
+    }
+    PMIx_generate_ppn(ranks, &ppn);
+    free(ranks);
+    pmix_strncpy(info[5].key, PMIX_PROC_MAP, PMIX_MAX_KEYLEN);
+    info[5].value.type = PMIX_STRING;
+    info[5].value.data.string = strdup(ppn);
+
+    pmix_strncpy(info[6].key, PMIX_JOB_SIZE, PMIX_MAX_KEYLEN);
+    info[6].value.type = PMIX_UINT32;
+    info[6].value.data.uint32 = univ_size;
+
+    pmix_strncpy(info[7].key, PMIX_APPNUM, PMIX_MAX_KEYLEN);
+    info[7].value.type = PMIX_UINT32;
+    info[7].value.data.uint32 = getpid ();
+
+    int in_progress = 1, rc;
+    if (PMIX_SUCCESS == (rc = PMIx_server_register_nspace(name, local_size,
+                                    info, ninfo, release_cb, &in_progress))) {
+        PMIX_WAIT_FOR_COMPLETION(in_progress);
+    }
+    PMIX_INFO_FREE(info, ninfo);
+}
+
 static void server_unpack_procs(char *buf, size_t size)
 {
     char *ptr = buf;
@@ -173,7 +187,7 @@ static void server_unpack_procs(char *buf, size_t size)
     char *nspace;
 
     while ((size_t)(ptr - buf) < size) {
-        ns_count = (size_t)*ptr;
+        ns_count = *(size_t *)ptr;
         ptr += sizeof(size_t);
 
         for (i = 0; i < ns_count; i++) {
@@ -181,16 +195,16 @@ static void server_unpack_procs(char *buf, size_t size)
             size_t ltasks, ntasks;
             int server_id;
 
-            server_id = *ptr;
+            server_id = *(int *)ptr;
             ptr += sizeof(int);
 
             nspace = ptr;
             ptr += PMIX_MAX_NSLEN+1;
 
-            ntasks = (size_t)*ptr;
+            ntasks = *(size_t *)ptr;
             ptr += sizeof(size_t);
 
-            ltasks = (size_t)*ptr;
+            ltasks = *(size_t *)ptr;
             ptr += sizeof(size_t);
 
             PMIX_LIST_FOREACH(tmp, server_nspace, server_nspace_t) {
@@ -212,7 +226,7 @@ static void server_unpack_procs(char *buf, size_t size)
             }
             size_t i;
             for (i = 0; i < ltasks; i++) {
-                int rank = (int)*ptr;
+                int rank = *(int *)ptr;
                 ptr += sizeof(int);
                 if (ns_item->task_map[rank] >= 0) {
                     continue;
@@ -459,7 +473,7 @@ static void _libpmix_cb(void *cbdata)
     }
 }
 
-static void server_read_cb(evutil_socket_t fd, short event, void *arg)
+static void server_read_cb(int fd, short event, void *arg)
 {
     server_info_t *server = (server_info_t*)arg;
     msg_hdr_t msg_hdr;
@@ -491,8 +505,8 @@ static void server_read_cb(evutil_socket_t fd, short event, void *arg)
     switch(msg_hdr.cmd) {
         case CMD_BARRIER_REQUEST:
             barrier_cnt++;
-            TEST_VERBOSE(("CMD_BARRIER_REQ req from %d cnt %d", msg_hdr.src_id,
-                          barrier_cnt));
+            TEST_VERBOSE(("CMD_BARRIER_REQ req from %d cnt %lu", msg_hdr.src_id,
+                          (unsigned long)barrier_cnt));
             if (pmix_list_get_size(server_list) == barrier_cnt) {
                 barrier_cnt = 0; /* reset barrier counter */
                 server_info_t *tmp_server;
@@ -521,8 +535,8 @@ static void server_read_cb(evutil_socket_t fd, short event, void *arg)
                 msg_buf = NULL;
             }
 
-            TEST_VERBOSE(("CMD_FENCE_CONTRIB req from %d cnt %d size %d",
-                        msg_hdr.src_id, contrib_cnt, msg_hdr.size));
+            TEST_VERBOSE(("CMD_FENCE_CONTRIB req from %d cnt %lu size %d",
+                        msg_hdr.src_id, (unsigned long)contrib_cnt, msg_hdr.size));
             if (pmix_list_get_size(server_list) == contrib_cnt) {
                 server_info_t *tmp_server;
                 PMIX_LIST_FOREACH(tmp_server, server_list, server_info_t) {
@@ -533,8 +547,8 @@ static void server_read_cb(evutil_socket_t fd, short event, void *arg)
                     resp_hdr.size = fence_buf_offset;
                     server_send_msg(&resp_hdr, fence_buf, fence_buf_offset);
                 }
-                TEST_VERBOSE(("CMD_FENCE_CONTRIB complete, size %d",
-                              fence_buf_offset));
+                TEST_VERBOSE(("CMD_FENCE_CONTRIB complete, size %lu",
+                              (unsigned long)fence_buf_offset));
                 if (fence_buf) {
                     free(fence_buf);
                     fence_buf = NULL;
@@ -637,13 +651,13 @@ static void server_unpack_dmdx(char *buf, int *sender, pmix_proc_t *proc)
 {
     char *ptr = buf;
 
-    *sender = (int)*ptr;
+    *sender = *(int *)ptr;
     ptr += sizeof(int);
 
     memcpy(proc->nspace, ptr, PMIX_MAX_NSLEN +1);
     ptr += PMIX_MAX_NSLEN +1;
 
-    proc->rank = (int)*ptr;
+    proc->rank = *(int *)ptr;
     ptr += sizeof(int);
 }
 
@@ -657,6 +671,8 @@ static void _dmdx_cb(int status, char *data, size_t sz, void *cbdata)
     msg_hdr.src_id = my_server_id;
     msg_hdr.size = sz;
     msg_hdr.dst_id = *sender_id;
+    TEST_VERBOSE(("srv #%d: DMDX RESPONSE: receiver=%d, size=%lu,",
+                  my_server_id, *sender_id, (unsigned long)sz));
     free(sender_id);
 
     server_send_msg(&msg_hdr, data, sz);
@@ -790,9 +806,9 @@ int server_init(test_params *params)
     if (params->nservers && pmix_list_get_size(server_list)) {
         server_info_t *server;
         PMIX_LIST_FOREACH(server, server_list, server_info_t) {
-            server->evread = event_new(pmix_globals.evbase, server->rd_fd,
+            server->evread = pmix_event_new(pmix_globals.evbase, server->rd_fd,
                               EV_READ|EV_PERSIST, server_read_cb, server);
-            event_add(server->evread, NULL);
+            pmix_event_add(server->evread, NULL);
         }
     }
 
@@ -878,14 +894,8 @@ int server_launch_clients(int local_size, int univ_size, int base_rank,
                   univ_size));
 
     TEST_VERBOSE(("Setting job info"));
-    fill_seq_ranks_array(local_size, base_rank, &ranks);
-    if (NULL == ranks) {
-        PMIx_server_finalize();
-        TEST_ERROR(("fill_seq_ranks_array failed"));
-        return PMIX_ERROR;
-    }
     (void)snprintf(proc.nspace, PMIX_MAX_NSLEN, "%s-%d", TEST_NAMESPACE, num_ns);
-    set_namespace(local_size, univ_size, ranks, proc.nspace);
+    set_namespace(local_size, univ_size, base_rank, proc.nspace);
     if (NULL != ranks) {
         free(ranks);
     }
@@ -909,18 +919,19 @@ int server_launch_clients(int local_size, int univ_size, int base_rank,
     /* fork/exec the test */
     for (n = 0; n < local_size; n++) {
         proc.rank = base_rank + rank_counter;
+        rc = PMIx_server_register_client(&proc, myuid, mygid, NULL, NULL, NULL);
+        if (PMIX_SUCCESS != rc && PMIX_OPERATION_SUCCEEDED != rc) {
+            TEST_ERROR(("Server register client failed with error %d", rc));
+            PMIx_server_finalize();
+            cli_kill_all();
+            return 0;
+        }
         if (PMIX_SUCCESS != (rc = PMIx_server_setup_fork(&proc, client_env))) {//n
             TEST_ERROR(("Server fork setup failed with error %d", rc));
             PMIx_server_finalize();
             cli_kill_all();
             return rc;
         }
-        if (PMIX_SUCCESS != (rc = PMIx_server_register_client(&proc, myuid, mygid, NULL, NULL, NULL))) {//n
-            TEST_ERROR(("Server fork setup failed with error %d", rc));
-            PMIx_server_finalize();
-            cli_kill_all();
-            return 0;
-        }
 
         cli_info[cli_counter].pid = fork();
         if (cli_info[cli_counter].pid < 0) {
